/* KmlcMatrix.hh */
/* Created by Enomoto Sanshiro on 18 January 2001 */
/* Last updated by Enomoto Sanshiro on 18 January 2001. */


#ifndef __KmlcMatrix_hh__
#define __KmlcMatrix_hh__


#include <complex>


template<class T = double> class TKmlcMatrixBase {
  public:
    inline TKmlcMatrixBase(void);
    inline TKmlcMatrixBase(int NumberOfRows, int NumberOfColumns);
    inline TKmlcMatrixBase(const TKmlcMatrixBase<T>& Matrix);
    inline ~TKmlcMatrixBase();
    inline TKmlcMatrixBase<T>& operator=(const TKmlcMatrixBase<T> Matrix);
    inline void Resize(int NumberOfRows, int NumberOfColumns);
    inline int NumberOfRows(void) const;
    inline int NumberOfColumns(void) const;
    inline T* RawArray(void);
    inline const T* RawArray(void) const;
    inline T** RawMatrix(void);
    inline const T** RawMatrix(void) const;
  public:
    inline T* operator[] (int RowIndex);
    inline const T* operator[] (int RowIndex) const;
    inline operator T*();
    inline operator const T*() const;
    inline operator T**();
    inline operator const T**() const;
  protected:
    int _NumberOfRows, _NumberOfColumns;
    int _Capacity;
    T* _RawArray;
    mutable T** _RawMatrix;
};


typedef TKmlcMatrixBase<double> TKmlcMatrix;
typedef TKmlcMatrixBase<std::complex<double> > TKmlcComplexMatrix;


template<class T> inline TKmlcMatrixBase<T>::TKmlcMatrixBase(void)
{
    _Capacity = 0;
    _RawArray = 0;
    _RawMatrix = 0;
    
    _NumberOfRows = 0;
    _NumberOfColumns = 0;
}

template<class T> inline TKmlcMatrixBase<T>::TKmlcMatrixBase(int NumberOfRows, int NumberOfColumns)
{
    _Capacity = 0;
    _RawArray = 0;
    _RawMatrix = 0;

    Resize(NumberOfRows, NumberOfColumns);
}

template<class T> inline TKmlcMatrixBase<T>::TKmlcMatrixBase(const TKmlcMatrixBase<T>& Matrix)
{
    _Capacity = 0;
    _RawArray = 0;
    _RawMatrix = 0;

    Resize(Matrix._NumberOfRows, Matrix._NumberOfColumns);

    for (unsigned i = 0; i < _NumberOfRows * _NumberOfColumns; i++) {
	_RawArray[i] = Matrix.RawArray[i];
    }
}

template<class T> inline TKmlcMatrixBase<T>::~TKmlcMatrixBase()
{
    delete[] _RawMatrix;
    delete[] _RawArray;
}

template<class T> inline TKmlcMatrixBase<T>& TKmlcMatrixBase<T>::operator=(const TKmlcMatrixBase<T> Matrix)
{
    Resize(Matrix._NumberOfRows, Matrix._NumberOfColumns);

    for (unsigned i = 0; i < _NumberOfRows * _NumberOfColumns; i++) {
	_RawArray[i] = Matrix.RawArray[i];
    }
}

template<class T> inline void TKmlcMatrixBase<T>::Resize(int NumberOfRows, int NumberOfColumns)
{
    _NumberOfRows = NumberOfRows;
    _NumberOfColumns = NumberOfColumns;

    if (_Capacity < _NumberOfRows * _NumberOfColumns) {
	delete[] _RawArray;
	_Capacity = _NumberOfRows * _NumberOfColumns;
	_RawArray = new T[_Capacity];
    }

    delete[] _RawMatrix;
    _RawMatrix = 0;
}

template<class T> inline int TKmlcMatrixBase<T>::NumberOfRows(void) const
{
    return _NumberOfRows;
}

template<class T> inline int TKmlcMatrixBase<T>::NumberOfColumns(void) const
{
    return _NumberOfColumns;
}

template<class T> inline T* TKmlcMatrixBase<T>::RawArray(void)
{
    return _RawArray;
}

template<class T> inline const T* TKmlcMatrixBase<T>::RawArray(void) const
{
    return _RawArray;
}

template<class T> inline T** TKmlcMatrixBase<T>::RawMatrix(void)
{
    if (_RawMatrix == 0) {
	_RawMatrix = new (T*)[_NumberOfRows];
	for (int Row = 0; Row < _NumberOfRows; Row++) {
	    _RawMatrix[Row] = _RawArray + Row * _NumberOfColumns;
	}
    }

    return _RawMatrix;
}

template<class T> inline const T** TKmlcMatrixBase<T>::RawMatrix(void) const
{
    if (_RawMatrix == 0) {
	_RawMatrix = new (T*)[_NumberOfRows];
	for (int Row = 0; Row < _NumberOfRows; Row++) {
	    _RawMatrix[Row] = _RawArray + Row * _NumberOfColumns;
	}
    }

    return _RawMatrix;
}

template<class T> inline T* TKmlcMatrixBase<T>::operator[] (int RowIndex)
{
    return _RawArray + RowIndex * _NumberOfColumns;
}

template<class T> inline const T* TKmlcMatrixBase<T>::operator[] (int RowIndex) const
{
    return _RawArray + RowIndex * _NumberOfColumns;
}

template<class T> inline TKmlcMatrixBase<T>::operator T*()
{
    return _RawArray;
}

template<class T> inline TKmlcMatrixBase<T>::operator const T*() const
{
    return _RawArray;
}

template<class T> inline TKmlcMatrixBase<T>::operator T**()
{
    return RawMatrix();
}

template<class T> inline TKmlcMatrixBase<T>::operator const T**() const
{
    return RawMatrix();
}

#endif
