/* KinokoCanvasDataScale.cc */
/* Created by Enomoto Sanshiro on 11 July 2000. */
/* Last updated by Enomoto Sanshiro on 7 December 2003. */


#include <iostream>
#include <algorithm>
#include <cmath>
#include <climits>
#include "KinokoCanvasDataScale.hh"

using namespace std;


TKinokoCanvasDataScale::TKinokoCanvasDataScale(float Min, float Max, int MaxNumberOfDivisions)
{
    _Min = min(Min, Max);
    _Max = max(Min, Max);
    _MaxNumberOfDivisions = MaxNumberOfDivisions;

    _Width = _Max - _Min;
    _FirstDivision = _Min;
    _LastDivision = _Max;
    _DivisionStep = -1;
    _NumberOfDivisions = -1;
    _TickStep = -1;
    _NumberOfTicks = -1;

    _UsedMin = _Min;
    _UsedMax = _Max;
}

TKinokoCanvasDataScale::~TKinokoCanvasDataScale()
{
}

float TKinokoCanvasDataScale::Min(void) const
{
    return _Min;
}

float TKinokoCanvasDataScale::Max(void) const
{
    return _Max;
}

float TKinokoCanvasDataScale::Width(void) const
{
    return _Width;
}

TKinokoCanvasDataScale& TKinokoCanvasDataScale::Rescale(float Min, float Max, int MaxNumberOfDivisions)
{
    _Min = min(Min, Max);
    _Max = max(Min, Max);
    _MaxNumberOfDivisions = MaxNumberOfDivisions;

    _Width = Max - Min;

    // recalculate steps only when the range has been changed //
    if (
	(fabs(_Min - _UsedMin) > _Width * 1.0e-5) ||
	(fabs(_Max - _UsedMax) > _Width * 1.0e-5)
    ){
	_FirstDivision = _Min;
	_LastDivision = _Max;
	_DivisionStep = -1;
	_NumberOfDivisions = -1;
	_NumberOfTicks = -1;
	_UsedMin = _Min;
	_UsedMax = _Max;
    }

    return *this;
}

int TKinokoCanvasDataScale::NumberOfDivisions(void)
{
    if (_NumberOfDivisions < 0) {
	pair<float, float> StepData = DecideSteps(_Min, _Max, _MaxNumberOfDivisions);
	_FirstDivision = StepData.first;
	_DivisionStep = StepData.second;

	_NumberOfDivisions = (int) ((_Max - _FirstDivision) / _DivisionStep) + 1;
	_LastDivision = _FirstDivision + (_NumberOfDivisions - 1) * _DivisionStep;
    }

    return _NumberOfDivisions;
}

float TKinokoCanvasDataScale::DivisionValueOf(int DivisionIndex)
{
    if (_NumberOfDivisions < 0) {
	NumberOfDivisions();
    }

    return _FirstDivision + DivisionIndex * _DivisionStep;
}

int TKinokoCanvasDataScale::NumberOfTicks(void)
{
    if (_NumberOfTicks >= 0) {
	return _NumberOfTicks;
    }

    if (_NumberOfDivisions < 0) {
	NumberOfDivisions();
    }
    
    if (_NumberOfTicks < 0) {
	int Exponent = (int) log10(1.01 * _DivisionStep);
	if (_DivisionStep < 0.99) {
	    Exponent -= 1;
	}
	int Factor = (int) ((1.01 * _DivisionStep) / pow(10.0, Exponent));
	if (Factor == 1) {
	    Factor = 5;
	}
	else if (Factor == 2) {
	    Factor = 4;
	}
	else if (Factor > 5) {
	    Factor /= 2;
	}

	_TickStep = _DivisionStep / Factor;
	_NumberOfTicks = (_NumberOfDivisions - 1) * Factor + 1;
	int NumberOfLeadingTicks = (int) ((_FirstDivision - _Min) / _TickStep);
	int NumberOfTrailingTicks = (int) ((_Max - _LastDivision) / _TickStep);
	_FirstTick = _FirstDivision - _TickStep * NumberOfLeadingTicks;
	_NumberOfTicks += NumberOfLeadingTicks + NumberOfTrailingTicks;
    }

    return _NumberOfTicks;
}

float TKinokoCanvasDataScale::TickValueOf(int TickIndex)
{
    if (_NumberOfTicks < 0) {
	NumberOfTicks();
    }

    return _FirstTick + TickIndex * _TickStep;
}

pair<float, float> TKinokoCanvasDataScale::DecideSteps(float Min, float Max, int MaxNumberOfSteps)
{
    int NumberOfSteps = max(2, MaxNumberOfSteps / 2);

    pair<float, float> Step;  // pair of first-step-value and step-width
    float Width = Max - Min;

    // order-normalization to make the width fit in range [N/2, 20*N]
    if (Width > 20 * NumberOfSteps) {
	Step = DecideSteps(Min / 10, Max / 10, MaxNumberOfSteps);
	Step.first *= 10;
	Step.second *= 10;
	return Step;
    }
    else if (Width < NumberOfSteps / 2.0) {
	Step = DecideSteps(Min * 10, Max * 10, MaxNumberOfSteps);
	Step.first /= 10;
	Step.second /= 10;
	return Step;
    }

    // factor-normalization(5) to make the width fit in range [N/2, 5*N)
    else if (Width > 4.99 * NumberOfSteps) {
	// I*5 normalization: once at most //
	// here the upper limit should be greater than 5*N to avoid
	// subsequent I*0.5 normalization which result in ugly 
	// I*2.5 normalization
	Step = DecideSteps(Min / 5, Max / 5, MaxNumberOfSteps);
	Step.first *= 5;
	Step.second *= 5;
	return Step;
    }

    // factor-normalization(2) to make the width fit in range (N, 2*N]
    else if (Width > 2.01 * NumberOfSteps) {
	// I*2 normalization: twice at most //
	Step = DecideSteps(Min / 2, Max / 2, MaxNumberOfSteps);
	Step.first *= 2;
	Step.second *= 2;
	return Step;
    }
    else if (Width < 0.99 * NumberOfSteps) {
	// I*0.5 normalization: once at most //
	Step = DecideSteps(Min * 2, Max * 2, MaxNumberOfSteps);
	Step.first /= 2;
	Step.second /= 2;
	return Step;
    }

    if ((Min < -LONG_MAX) || (Min > LONG_MAX)) {
	Step.first = Min;
    }
    else {
	Step.first = ((long) Min < Min) ? ((long) Min + 1) : ((long) Min);
    }

    Step.second = 1;

    return Step;
}



TKinokoCanvasLogDataScale::TKinokoCanvasLogDataScale(float Min, float Max, int NumberOfDivisions)
: TKinokoCanvasDataScale(Min, Max, NumberOfDivisions)
{
    _IsUsingNormalScale = false;
    
    //...  to be removed
    if ((Min <= 0) || (Max <= Min)) {
	cerr << "WARNING: TKinokoCanvasLogDataScale::TKinokoCavansLogDataScale(): ";
	cerr << "invalid log scale range: ";
	cerr << "(" << Min << ", " << Max << ")" << endl;
    }
    if (_Min <= 0) {
	_Min = 0.5;
    }
    if (_Max <= _Min) {
	_Min = _Max / 100.0;
    }
}

TKinokoCanvasLogDataScale::~TKinokoCanvasLogDataScale()
{
}

TKinokoCanvasDataScale& TKinokoCanvasLogDataScale::Rescale(float Min, float Max, int NumberOfDivisions)
{
    //...  to be removed
    if ((Min <= 0) || (Max <= Min)) {
	cerr << "WARNING: TKinokoCanvasLogDataScale::Rescale(): ";
	cerr << "invalid log scale range: ";
	cerr << "(" << Min << ", " << Max << ")" << endl;
    }
    if (Min <= 0) {
	Min = 0.5;
    }
    if (Max <= Min) {
	Min = Max / 10.0;
    }

    return TKinokoCanvasDataScale::Rescale(Min, Max, NumberOfDivisions);
}

int TKinokoCanvasLogDataScale::NumberOfDivisions(void)
{
    if (_NumberOfDivisions >= 0) {
	return _NumberOfDivisions;
    }

    float LogMin = log10(_Min);
    float LogMax = log10(_Max);

    int ILogMin = (int) (((int) LogMin < LogMin) ? (LogMin + 1) : LogMin);
    float LogWidth = LogMax - ILogMin;
    
    if (LogWidth > 1) {
	int LogDivisionStep = (int) ((LogWidth-1) / _MaxNumberOfDivisions) + 1;
	_NumberOfDivisions = (int) (LogWidth / LogDivisionStep) + 1;
	_DivisionStep = pow(10.0, (double) LogDivisionStep);
	_FirstDivision = pow(10.0, (double) ILogMin);
	_LastDivision = _FirstDivision * pow(_DivisionStep, _NumberOfDivisions - 1);
	_IsUsingNormalScale = false;
    }
    else {
	_NumberOfDivisions = -1;
	_NumberOfDivisions = TKinokoCanvasDataScale::NumberOfDivisions();
	_IsUsingNormalScale = true;
    }

    return _NumberOfDivisions;
}

float TKinokoCanvasLogDataScale::DivisionValueOf(int DivisionIndex)
{
    if (_NumberOfDivisions < 0) {
	NumberOfDivisions();
    }

    if (_IsUsingNormalScale) {
	return TKinokoCanvasDataScale::DivisionValueOf(DivisionIndex);
    }

    return _FirstDivision * pow(_DivisionStep, DivisionIndex);
}

int TKinokoCanvasLogDataScale::NumberOfTicks(void)
{
    if (_NumberOfTicks >= 0) {
	return _NumberOfTicks;
    }

    if (_NumberOfDivisions < 0) {
	NumberOfDivisions();
    }

    if (! _IsUsingNormalScale) {
	double OrderOfMin = floor(log10(1.01 * _Min));
	double OrderOfMax = floor(log10(1.01 * _Max));
	double FactorOfMin = ceil(0.99 * (_Min / pow(10.0, OrderOfMin)));
	double FactorOfMax = floor(1.01 * (_Max / pow(10.0, OrderOfMax)));

	_TickIndexOffset = (int) ((FactorOfMin - 1) + 0.1);
	_FirstTick = pow(10.0, OrderOfMin);
	_NumberOfTicks = (int) (9 * (OrderOfMax - OrderOfMin) + (FactorOfMax - 1) + 0.1) - _TickIndexOffset;
    }
    else {
	_NumberOfTicks = -1;
	_NumberOfTicks = TKinokoCanvasDataScale::NumberOfTicks();
    }

    return _NumberOfTicks;
}

float TKinokoCanvasLogDataScale::TickValueOf(int TickIndex)
{
    if (_NumberOfTicks < 0) {
	NumberOfTicks();
    }

    if (_IsUsingNormalScale) {
	return TKinokoCanvasDataScale::TickValueOf(TickIndex);
    }

    TickIndex += _TickIndexOffset;

    return (TickIndex % 9 + 1) * _FirstTick * pow(10.0, TickIndex / 9);
}
