/* KinokoCanvasPlotObject.cc */
/* Created by Enomoto Sanshiro on 11 July 2000. */
/* Last updated by Enomoto Sanshiro on 12 June 2004. */


#include <sstream>
#include <algorithm>
#include <cmath>
#include "KinokoCanvasDataScale.hh"
#include "KinokoCanvasColorScale.hh"
#include "KinokoCanvasFramedObject.hh"
#include "KinokoCanvasPlotObject.hh"

using namespace std;


#define clip(x, lower, upper) (min(max((lower), (x)), (upper)))


TKinokoCanvasPlotObject::TKinokoCanvasPlotObject(TKinokoCanvas* Canvas, TKinokoCanvasImageArea* ImageArea)
: TKinokoCanvasFramedObject(Canvas, ImageArea)
{
    _BottomMargin += (int) (2.2 * _LabelCharacterHeight);
    _RightMargin += (int) (1.5 * _LabelCharacterWidth);
    _LeftMargin += (int) (4.5 * _LabelCharacterWidth);

    _IsXScaleLog = false;
    _IsYScaleLog = false;

    _IsXScaleLabelDisabled = false;
    _IsYScaleLabelDisabled = false;
    _IsYTitleVertical = false;

    _NumberOfXScaleLabels = 0;
    _NumberOfYScaleLabels = 0;
    SetCoordinate(0, 0, 1, 1);
    _XScale = new TKinokoCanvasDataScale(0, 1, _NumberOfXScaleLabels);
    _YScale = new TKinokoCanvasDataScale(0, 1, _NumberOfYScaleLabels);
}

TKinokoCanvasPlotObject::~TKinokoCanvasPlotObject()
{
    delete _YScale;
    delete _XScale;
}

TKinokoShellObject* TKinokoCanvasPlotObject::Clone(void)
{
    return new TKinokoCanvasPlotObject(_Canvas, _ImageArea);
}

vector<string> TKinokoCanvasPlotObject::ActionList(void)
{
    vector<string> ActionList = TKinokoCanvasFramedObject::ActionList();

    ActionList.push_back("setYScaleLinear");
    ActionList.push_back("setYScaleLog");

    return ActionList;
}

int TKinokoCanvasPlotObject::ProcessAction(const string& ActionName)
{
    int Result = 1;
    if (ActionName == "setYScaleLinear") {
	SetYScaleLinear();
    }
    else if (ActionName == "setYScaleLog") {
	SetYScaleLog();
    }
    else {
	Result = TKinokoCanvasFramedObject::ProcessAction(ActionName);
    }

    return Result;
}

int TKinokoCanvasPlotObject::ProcessCommand(const string& Command, istream& InputStream)
{
    int Result = 0;

    if (Command == "plot") {
	Result = ProcessPlotCommand(InputStream, Style_Points | Style_Lines);
    }
    else if (Command == "linesplot") {
	Result = ProcessPlotCommand(InputStream, Style_Lines);
    }
    else if (Command == "pointsplot") {
	Result = ProcessPlotCommand(InputStream, Style_Points);
    }
    else if (Command == "filledplot") {
	Result = ProcessPlotCommand(InputStream, Style_Filled);
    }
    else if (Command == "errorplot") {
	Result = ProcessErrorPlotCommand(InputStream);
    }
    else if (Command == "minmaxplot") {
	Result = ProcessMinmaxPlotCommand(InputStream);
    }
    else if (Command == "hist") {
	Result = ProcessHistCommand(InputStream);
    }
    else if (Command == "histfill") {
	Result = ProcessHistCommand(InputStream, Style_Filled);
    }
    else if (Command == "comment") {
	Result = ProcessCommentCommand(InputStream);
    }
    else if (Command == "drawxscalelabel") {
	Result = ProcessDrawXScaleLabelCommand(InputStream);
    }
    else if (Command == "drawyscalelabel") {
	Result = ProcessDrawYScaleLabelCommand(InputStream);
    }
    else {
	return TKinokoCanvasFramedObject::ProcessCommand(Command, InputStream);
    }

    return Result;
}

int TKinokoCanvasPlotObject::ProcessSetCommand(const string& Name, const string& Value)
{
    int Result = 1;

    if (Name == "xscale") {
	if (Value == "log") {
	    SetXScaleLog();
	}
	else {
	    SetXScaleLinear();
	}
    }
    else if (Name == "yscale") {
	if (Value == "log") {
	    SetYScaleLog();
	}
	else {
	    SetYScaleLinear();
	}
    }
    else if (Name == "xtitle") {
	_XTitle = Value;
    }
    else if (Name == "ytitle") {
	_YTitle = Value;
    }
    else if (Name == "xscalelabel") {
	_IsXScaleLabelDisabled = (Value == "disabled");
    }
    else if (Name == "yscalelabel") {
	_IsYScaleLabelDisabled = (Value == "disabled");
    }
    else if (Name == "ytitleorientation") {
	//... temporary
	bool IsOldYTitleVertical = _IsYTitleVertical;
	_IsYTitleVertical = (Value == "vertical");
	if ((! IsOldYTitleVertical) && _IsYTitleVertical) {
	    _LeftMargin += 2 * _LabelCharacterWidth;
	    _RightMargin += 2 * _LabelCharacterWidth;
	}
	else if (IsOldYTitleVertical && (! _IsYTitleVertical)) {
	    _LeftMargin -= 2 * _LabelCharacterWidth;
	    _RightMargin -= 2 * _LabelCharacterWidth;
	}
	SetPosition(_OffsetX, _OffsetY, _Width, _Height);
    }
    else {
	Result = TKinokoCanvasFramedObject::ProcessSetCommand(Name, Value);
    }

    return Result;
}

float TKinokoCanvasPlotObject::CanvasXOf(float x)
{
    if (! _IsXScaleLog) {
	return (x - _XMin) / _XWidth * _FrameWidth + _FrameOffsetX;
    }
    else {
	return (log10(x) - _LogXMin) / _LogXWidth * _FrameWidth + _FrameOffsetX;
    }
}

float TKinokoCanvasPlotObject::CanvasYOf(float y)
{
    if (! _IsYScaleLog) {
	return (1.0 - (y - _YMin) / _YWidth) * _FrameHeight + _FrameOffsetY;
    }
    else {
	return (1.0 - (log10(y) - _LogYMin) / _LogYWidth) * _FrameHeight + _FrameOffsetY;
    }
}

int TKinokoCanvasPlotObject::SetCoordinate(float XMin, float YMin, float XWidth, float YWidth)
{
    _XMinOrg = XMin;
    _XWidthOrg = XWidth;
    _YMinOrg = YMin;
    _YWidthOrg = YWidth;

    if (_IsXScaleLog) {
	if (XMin <= 0) {
	    float XMax = XMin + XWidth;
	    float XMaxForLog = (XMax <= 0) ? 1.0 : XMax;
	    float XMinForLog = min(XMaxForLog / 100.0, 0.5);
	    XMin = XMinForLog;
	    XWidth = XMaxForLog - XMinForLog;
	}

	_LogXMin = log10(XMin);
	_LogXMax = log10(XMin + XWidth);
	_LogXWidth = _LogXMax - _LogXMin;
    }
    
    if (_IsYScaleLog) {
	if (YMin <= 0) {
	    float YMax = YMin + YWidth;
	    float YMaxForLog = (YMax <= 0) ? 1.0 : YMax;
	    float YMinForLog = min(YMaxForLog / 100.0, 0.5);
	    YMin = YMinForLog;
	    YWidth = YMaxForLog - YMinForLog;
	}

	_LogYMin = log10(YMin);
	_LogYMax = log10(YMin + YWidth);
	_LogYWidth = _LogYMax - _LogYMin;
    }

    TKinokoCanvasFramedObject::SetCoordinate(XMin, YMin, XWidth, YWidth);

    return 1;
}

int TKinokoCanvasPlotObject::SetXScaleLinear(void)
{
    if (_IsXScaleLog) {
	_IsXScaleLog = false;
	SetCoordinate(_XMinOrg, _XWidthOrg, _YMinOrg, _YWidthOrg);
	delete _XScale;
	_XScale = new TKinokoCanvasDataScale(
	    _XMin, _XMax, _NumberOfXScaleLabels
	);
    }

    return 1;
}

int TKinokoCanvasPlotObject::SetXScaleLog(void)
{
    if (! _IsXScaleLog) {
	_IsXScaleLog = true;
	SetCoordinate(_XMinOrg, _XWidthOrg, _YMinOrg, _YWidthOrg);
	delete _XScale;
	_XScale = new TKinokoCanvasLogDataScale(
	    _XMin, _XMax, _NumberOfXScaleLabels
	);
    }

    return 1;
}

int TKinokoCanvasPlotObject::SetYScaleLinear(void)
{
    if (_IsYScaleLog) {
	_IsYScaleLog = false;
	SetCoordinate(_XMinOrg, _XWidthOrg, _YMinOrg, _YWidthOrg);
	delete _YScale;
	_YScale = new TKinokoCanvasDataScale(
	    _YMin, _YMax, _NumberOfYScaleLabels
	);
    }

    return 1;
}

int TKinokoCanvasPlotObject::SetYScaleLog(void)
{
    if (! _IsYScaleLog) {
	_IsYScaleLog = true;
	SetCoordinate(_XMinOrg, _XWidthOrg, _YMinOrg, _YWidthOrg);
	delete _YScale;
	_YScale = new TKinokoCanvasLogDataScale(
	    _YMin, _YMax, _NumberOfYScaleLabels
	);
    }

    return 1;
}

int TKinokoCanvasPlotObject::ProcessPlotCommand(istream& InputStream, int Style)
{
    if (! _IsFrameValid) {
	return 1;
    }

    int Result = 0;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndex);
    int OldLineWidth = _ImageArea->SetLineWidth(_LineWidth);
    int OldLineStyleIndex = _ImageArea->SetLineStyle(_LineStyleIndex);

    float x, y;
    float cx, cy;
    vector<pair<float, float> > PointList;
    while (InputStream >> x >> y) {
	//... temporary
	if ((x < _XMin) || (x > _XMax)) {
	    continue;
	}
	y = clip(y, _YMin, _YMax);

	cx = CanvasXOf(x);
	cy = CanvasYOf(y);
	
	if (Style & Style_Points) {
	    if (_Marker) {
		_Marker->Draw(_ImageArea, cx, cy, _MarkerSize);
	    }
	    else {
		_ImageArea->DrawRect(cx - 2, cy - 2, cx + 2, cy + 2);
	    }
	}
	if (Style & (Style_Lines | Style_Filled)) {
	    PointList.push_back(make_pair(cx, cy));
	}
    }

    _ImageArea->SetLineStyle(_LineStyleIndex);

    if (! PointList.empty()) {
	if (Style & Style_Lines) {
	    _ImageArea->DrawLines(PointList);
	}
	else if (Style & Style_Filled) {
	    _ImageArea->DrawPolygonFill(PointList);
	}
    }

    _ImageArea->SetColor(OldColorIndex);
    _ImageArea->SetLineWidth(OldLineWidth);
    _ImageArea->SetLineStyle(OldLineStyleIndex);

    return Result = 1;
}

int TKinokoCanvasPlotObject::ProcessErrorPlotCommand(istream& InputStream)
{
    if (! _IsFrameValid) {
	return 1;
    }

    int Result = 0;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndex);
    int OldLineWidth = _ImageArea->SetLineWidth(_LineWidth);
    int OldLineStyleIndex = _ImageArea->SetLineStyle();

    float x, y, ex, ey;
    float cx, cy, cx0, cy0, cx1, cy1;
    while (InputStream >> x >> y >> ex >> ey) {
	//... temporary
	if ((x < _XMin) || (x > _XMax)) {
	    continue;
	}

	cx = CanvasXOf(clip(x, _XMin, _XMax));
	cx0 = CanvasXOf(clip(x - ex, _XMin, _XMax));
	cx1 = CanvasXOf(clip(x + ex, _XMin, _XMax));

	cy = CanvasYOf(clip(y, _YMin, _YMax));
	cy0 = CanvasYOf(clip(y - ey, _YMin, _YMax));
	cy1 = CanvasYOf(clip(y + ey, _YMin, _YMax));

	if (cx0 != cx1) {
	    _ImageArea->DrawLine(cx0, cy, cx1, cy);
	}
	if (cy0 != cy1) {
	    _ImageArea->DrawLine(cx, cy0, cx, cy1);
	}

	if (_Marker) {
	    _Marker->Draw(_ImageArea, cx, cy, _MarkerSize);
	}
	else if ((cx0 == cx1) || (cy0 == cy1)) {
	    _ImageArea->DrawRect(cx - 2, cy - 2, cx + 2, cy + 2);
	}
    }

    _ImageArea->SetColor(OldColorIndex);
    _ImageArea->SetLineWidth(OldLineWidth);
    _ImageArea->SetLineStyle(OldLineStyleIndex);

    return Result = 1;
}

int TKinokoCanvasPlotObject::ProcessMinmaxPlotCommand(istream& InputStream)
{
    if (! _IsFrameValid) {
	return 1;
    }

    int Result = 0;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndex);
    int OldLineWidth = _ImageArea->SetLineWidth(_LineWidth);
    int OldLineStyleIndex = _ImageArea->SetLineStyle();

    float x, y, ymin, ymax, deviation;
    float cx, cy, cymin, cymax, cy0, cy1;
    while (InputStream >> x >> y >> ymin >> ymax >> deviation) {
	//... temporary
	if ((x < _XMin) || (x > _XMax)) {
	    continue;
	}
	y = clip(y, _YMin, _YMax);

	cx = CanvasXOf(x);
	cy = CanvasYOf(y);
	cymin = CanvasYOf(clip(ymin, _YMin, _YMax));
	cymax = CanvasYOf(clip(ymax, _YMin, _YMax));
	cy0 = CanvasYOf(clip(y - deviation, _YMin, _YMax));
	cy1 = CanvasYOf(clip(y + deviation, _YMin, _YMax));

	_ImageArea->DrawRect(cx - 2, cy - 2, cx + 2, cy + 2);
	if (cy0 != cy1) {
	    _ImageArea->DrawLine(cx, cy0, cx, cy1);
	}

	_ImageArea->DrawLine(cx - 2, cymin - 2, cx + 2, cymin + 2);
	_ImageArea->DrawLine(cx - 2, cymin + 2, cx + 2, cymin - 2);
	_ImageArea->DrawLine(cx - 2, cymax - 2, cx + 2, cymax + 2);
	_ImageArea->DrawLine(cx - 2, cymax + 2, cx + 2, cymax - 2);
    }

    _ImageArea->SetColor(OldColorIndex);
    _ImageArea->SetLineWidth(OldLineWidth);
    _ImageArea->SetLineStyle(OldLineStyleIndex);

    return Result = 1;
}

int TKinokoCanvasPlotObject::ProcessHistCommand(istream& InputStream, int Style)
{
    // syntax: hist xmin step y0 y1 y2 y3 ...
    // [xmin, xmin + step) -> y0, [xmin + step, xmin + 2 step) -> y1, ...

    if (! _IsFrameValid) {
	return 1;
    }

    int Result = 0;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndex);
    int OldLineWidth = _ImageArea->SetLineWidth(_LineWidth);
    int OldLineStyleIndex = _ImageArea->SetLineStyle(_LineStyleIndex);

    float xmin, xstep;
    if (! (InputStream >> xmin >> xstep)) {
	return 0;
    }

    float cx0 = CanvasXOf(xmin);
    float cy0 = CanvasYOf(_YMin);
    vector<pair<float, float> > PointList;
    PointList.push_back(make_pair(cx0, cy0));

    float x = xmin, y;
    float cx1, cy1;
    while (InputStream >> y) {
	//... temporary
	x += xstep;
	if ((x < _XMin) || (x > _XMax)) {
	    continue;
	}
	y = clip(y, _YMin, _YMax);

	cx1 = CanvasXOf(x);
	cy1 = CanvasYOf(y);

	PointList.push_back(make_pair(cx0, cy1));
	PointList.push_back(make_pair(cx1, cy1));

	cx0 = cx1;
	cy0 = cy1;
    }

    cy1 = CanvasYOf(_YMin);
    PointList.push_back(make_pair(cx0, cy1));

    if (Style & Style_Filled) {
	float cx0 = CanvasXOf(xmin);
	float cy0 = CanvasYOf(_YMin);
	PointList.push_back(make_pair(cx0, cy0));
	_ImageArea->DrawPolygonFill(PointList);
    }
    else {
	_ImageArea->DrawLines(PointList);
    }

    _ImageArea->SetColor(OldColorIndex);
    _ImageArea->SetLineWidth(OldLineWidth);
    _ImageArea->SetLineStyle(OldLineStyleIndex);

    return Result = 1;
}

int TKinokoCanvasPlotObject::ProcessCommentCommand(istream& InputStream)
{
    static const int SideMargin = 7;

    if (! _IsFrameValid) {
	return 1;
    }

    int Result = 0;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndexList[Color_Foreground]);
    int OldLineWidth = _ImageArea->SetLineWidth(1);
    int OldLineStyleIndex = _ImageArea->SetLineStyle();
    int OldFontIndex = _ImageArea->SetFont(_FontIndexList[Font_Normal]);

    enum TTextType {
	TextType_Plain,
	TextType_Math,
	NumberOfTextTypes
    };

    float x, y;
    if (InputStream >> x >> y >> ws) {
	float x0 = fabs((x / 100.0) * _FrameWidth) + _FrameOffsetX;
	float y0 = fabs((y / 100.0) * _FrameHeight) + _FrameOffsetY;
	
	char Buffer[256];
	vector<string> TextList;
	vector<float> YOffsetList;
	vector<float> TextTypeList;
	float MaxWidth = 0, TotalHeight = 0;
	while (InputStream.getline(Buffer, sizeof(Buffer), '\n')) {
	    int Offset, TextType;
	    float Width;
	    if (strncmp(Buffer, "math:", 5) == 0) {
		TextType = TextType_Math;
		Offset = 5;
		const TKinokoCanvasTextComposition& Composition = (
		    _MathTextComposerList[Font_Label]->Compose(Buffer + Offset)
		);
		Width = Composition.Width();
	    }
	    else if (strncmp(Buffer, "plain:", 6) == 0) {
		TextType = TextType_Plain;
		Offset = 6;
		Width = _ImageArea->TextWidthOf(Buffer + Offset);
	    }
	    else {
		TextType = TextType_Plain;
		Offset = 0;
		Width = _ImageArea->TextWidthOf(Buffer + Offset);
	    }

	    MaxWidth = max(MaxWidth, Width);
	    TotalHeight += _LineHeight;

	    TextList.push_back(Buffer + Offset);
	    YOffsetList.push_back(TotalHeight);
	    TextTypeList.push_back(TextType);
	}
        MaxWidth += 2 * SideMargin;
	TotalHeight += SideMargin;

	if (x < 0) {
	    x0 = x0 - MaxWidth;
	}
	if (y < 0) {
	    y0 = y0 - TotalHeight;
	}

	_ImageArea->DrawRect(x0, y0, x0 + MaxWidth, y0 + TotalHeight);

	for (unsigned i = 0; i < TextList.size(); i++) {
	    if (TextTypeList[i] == TextType_Math) {
		_Typesetter->Typeset(
		    x0 + SideMargin, y0 + YOffsetList[i],
		    _MathTextComposerList[Font_Label]->Compose(
			TextList[i]
		    ), 
		    "bl"
		);
	    }
	    else {
		_ImageArea->DrawText(
		    x0 + SideMargin, y0 + YOffsetList[i], TextList[i], "bl"
		);
	    }
	}

	Result = 1;
    }

    _ImageArea->SetColor(OldColorIndex);
    _ImageArea->SetLineWidth(OldLineWidth);
    _ImageArea->SetLineStyle(OldLineStyleIndex);
    _ImageArea->SetFont(OldFontIndex);

    return Result;
}

int TKinokoCanvasPlotObject::ProcessDrawXScaleLabelCommand(istream& InputStream)
{
    if (! _IsFrameValid) {
	return 1;
    }

    int Result = 0;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndexList[Color_Foreground]);
    int OldLineWidth = _ImageArea->SetLineWidth(1);
    int OldLineStyleIndex = _ImageArea->SetLineStyle();
    int OldFontIndex = _ImageArea->SetFont(_FontIndexList[Font_Label]);

    float x;
    if ((InputStream >> x) && ((x >= _XMin) || (x <= _XMax))) {
	float cx = CanvasXOf(x);
	float y1 = _FrameOffsetY + _FrameHeight;

	string Label;
	InputStream >> ws;
	if (getline(InputStream, Label)) {
	    _ImageArea->DrawText(cx, y1 + 3, Label, "tc");
	}
	else {
	    //... use typesetter 
	    _ImageArea->DrawNumberText(cx, y1 + 3, x, "tc");
	}
	_ImageArea->DrawLine(cx, y1 - 8, cx, y1);

	Result = 1;
    }

    _ImageArea->SetColor(OldColorIndex);
    _ImageArea->SetLineWidth(OldLineWidth);
    _ImageArea->SetLineStyle(OldLineStyleIndex);
    _ImageArea->SetFont(OldFontIndex);

    return Result;
}

int TKinokoCanvasPlotObject::ProcessDrawYScaleLabelCommand(istream& InputStream)
{
    if (! _IsFrameValid) {
	return 1;
    }

    int Result = 0;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndexList[Color_Foreground]);
    int OldLineWidth = _ImageArea->SetLineWidth(1);
    int OldLineStyleIndex = _ImageArea->SetLineStyle();
    int OldFontIndex = _ImageArea->SetFont(_FontIndexList[Font_Label]);

    float y;
    if ((InputStream >> y) && ((y >= _YMin) || (y <= _YMax))) {
	float cy = CanvasYOf(y);
	float x0 = _FrameOffsetX;

	InputStream >> ws;
	string Label;
	if (getline(InputStream, Label)) {
	    _ImageArea->DrawText(x0 - 3, cy, Label, "cr");
	}
	else {
	    //... use typesetter 
	    _ImageArea->DrawNumberText(x0 - 3, cy, y, "cr");
	}
	_ImageArea->DrawLine(x0, cy, x0 + 8, cy);

	Result = 1;
    }

    _ImageArea->SetColor(OldColorIndex);
    _ImageArea->SetLineWidth(OldLineWidth);
    _ImageArea->SetLineStyle(OldLineStyleIndex);
    _ImageArea->SetFont(OldFontIndex);

    return Result;
}

int TKinokoCanvasPlotObject::DrawFrame(void)
{
    TKinokoCanvasFramedObject::DrawFrame();

    _NumberOfXScaleLabels = min(
	12, _FrameWidth / (7 * _LabelCharacterWidth)
    );
    _NumberOfXScaleLabels = max(3, _NumberOfXScaleLabels);

    _NumberOfYScaleLabels = min(
	_FrameHeight / (2 * _LabelCharacterHeight),
	(int) (((float) _FrameHeight / _FrameWidth) * _NumberOfXScaleLabels)
    );
    _NumberOfYScaleLabels = max(3, _NumberOfYScaleLabels);

    if (! _IsFrameValid) {
	return 1;
    }

    int x0 = _FrameOffsetX;
    int y0 = _FrameOffsetY;
    int x1 = _FrameOffsetX + _FrameWidth;
    int y1 = _FrameOffsetY + _FrameHeight;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndexList[Color_Foreground]);
    int OldLineWidth = _ImageArea->SetLineWidth(2);
    int OldLineStyleIndex = _ImageArea->SetLineStyle();
    int OldFontIndex = _ImageArea->SetFont(_FontIndexList[Font_Label]);

    _ImageArea->DrawRect(x0, y0, x1, y1);

    if (strncmp(_XTitle.c_str(), "math:", 5) == 0) {
	_Typesetter->Typeset(
	    x1, (y1 + 1.8 * _LabelCharacterHeight),
	    _MathTextComposerList[Font_Label]->Compose(
		_XTitle.substr(5, string::npos)
	    ), 
	    "tr"
	);
    }
    else {
	_ImageArea->DrawText(
	    x1, y1 + 1.8 * _LabelCharacterHeight, 
	    _XTitle, 
	    "tr"
	);
    }

    if (! _IsYTitleVertical) {
	_ImageArea->DrawText(
	    x0, y0 - _LabelCharacterHeight, 
	    _YTitle, 
	    "br"
	);
    }
    else {
	if (strncmp(_YTitle.c_str(), "math:", 5) == 0) {
	    _Typesetter->SetTextOrientation(90);
	    _Typesetter->Typeset(
		_OffsetX + 0.2 * _LabelCharacterHeight, y0,
		_MathTextComposerList[Font_Label]->Compose(
		    _YTitle.substr(5, string::npos)
		), 
		"tl"
	    );
	    _Typesetter->SetTextOrientation(0);
	}
	else {
	    int OldTextOrientation = _ImageArea->SetTextOrientation(90);
	    _ImageArea->DrawText(
		_OffsetX + 0.2 * _LabelCharacterHeight, y0, 
		_YTitle, 
		"tl"
	    );
	    _ImageArea->SetTextOrientation(OldTextOrientation);
	}
    }

    _ImageArea->SetLineWidth(1);
    if (! _IsXScaleLabelDisabled) {
	_XScale->Rescale(_XMin, _XMax, _NumberOfXScaleLabels);
	double Scaling = 1.0;
#if 1
	if (_IsYTitleVertical && (! _IsXScaleLog)) {
	    double RangeScale = max(fabs(_XMin), fabs(_XMax));
	    if ((RangeScale >= 1.0e+4) || (RangeScale <= 9.9e-2)) {
		double Order = floor(log10(1.01 * RangeScale));
		Scaling = pow(10, Order);
		if (fabs(RangeScale/Scaling) < 2) {
		    Order -= 1;
		    Scaling /= 10.0;
		}

		ostringstream os;
		os << "{#times}10^{" << Order << "}";
		string ScalingText = os.str();

		_Typesetter->Typeset(
		    x1 + 3, y1, 
		    _MathTextComposerList[Font_Label]->Compose(ScalingText),
		    "bl"
	        );
	    }
	}
#endif

	int NumberOfXScaleDivisions = _XScale->NumberOfDivisions();
	for (int i = 0; i < NumberOfXScaleDivisions; i++) {
	    float xs = _XScale->DivisionValueOf(i);
	    float x = CanvasXOf(xs);
	    _ImageArea->DrawLine(x, y1 - 8, x, y1);
#if 0
	    _ImageArea->DrawNumberText(x, y1 + 3, xs, "tc");
#else
	    if (_IsXScaleLog && (_XMin > 0) && (_XMax / _XMin > 100)) {
		_Typesetter->Typeset(
		    x, y1 + 3, 
		    _MathTextComposerList[Font_Label]->ComposePowerOf10(xs),
		    "tc"
		);
	    }
	    else {
		_Typesetter->Typeset(
		    x, y1 + 3, 
		    _MathTextComposerList[Font_Label]->ComposeNumber(xs/Scaling),
		    "tc"
		);
	    }
#endif
	}

	int NumberOfXScaleTicks = _XScale->NumberOfTicks();
	for (int i = 0; i < NumberOfXScaleTicks; i++) {
	    float xs = _XScale->TickValueOf(i);
	    float x = CanvasXOf(xs);
	    _ImageArea->DrawLine(x, y1 - 5, x, y1);
	}
    }

    if (! _IsYScaleLabelDisabled) {
	_YScale->Rescale(_YMin, _YMax, _NumberOfYScaleLabels);
	double Scaling = 1.0;
#if 1
	if (! _IsYScaleLog) {
	    double RangeScale = max(fabs(_YMin), fabs(_YMax));
	    if ((RangeScale >= 1.0e+4) || (RangeScale <= 9.9e-2)) {
		double Order = floor(log10(1.01 * RangeScale));
		Scaling = pow(10, Order);
		if (fabs(RangeScale/Scaling) < 2) {
		    Order -= 1;
		    Scaling /= 10.0;
		}

		ostringstream os;
		os << "{#times}10^{" << Order << "}";
		string ScalingText = os.str();

		_Typesetter->Typeset(
		    x0, y0 - 3, 
		    _MathTextComposerList[Font_Label]->Compose(ScalingText),
		    "bl"
	        );
	    }
	}
#endif

	int NumberOfYScaleDivisions = _YScale->NumberOfDivisions();
	for (int i = 0; i < NumberOfYScaleDivisions; i++) {
	    float ys = _YScale->DivisionValueOf(i);
	    float y = CanvasYOf(ys);
	    _ImageArea->DrawLine(x0, y, x0 + 8, y);
#if 0
	    _ImageArea->DrawNumberText(x0 - 3, y, ys, "cr");
#else
	    if (_IsYScaleLog && (_YMin > 0) && (_YMax / _YMin > 100)) {
		_Typesetter->Typeset(
		    x0 - 3, y, 
		    _MathTextComposerList[Font_Label]->ComposePowerOf10(ys),
		    "cr"
		);
	    }
	    else {
		_Typesetter->Typeset(
		    x0 - 3, y, 
		    _MathTextComposerList[Font_Label]->ComposeNumber(ys/Scaling),
		    "cr"
		);
	    }
#endif	    
	}
	
	int NumberOfYScaleTicks = _YScale->NumberOfTicks();
	for (int i = 0; i < NumberOfYScaleTicks; i++) {
	    float ys = _YScale->TickValueOf(i);
	    float y = CanvasYOf(ys);
	    _ImageArea->DrawLine(x0, y, x0 + 5, y);
	}
    }
	
    _ImageArea->SetColor(OldColorIndex);
    _ImageArea->SetLineWidth(OldLineWidth);
    _ImageArea->SetLineStyle(OldLineStyleIndex);
    _ImageArea->SetFont(OldFontIndex);

    return 1;
}



TKinokoCanvasMapPlotObject::TKinokoCanvasMapPlotObject(TKinokoCanvas* Canvas, TKinokoCanvasImageArea* ImageArea)
: TKinokoCanvasFramedObject(Canvas, ImageArea), TKinokoCanvasColorScaledObject(Canvas, ImageArea, new TKinokoCanvasGrayBackgroundRainbowColorScale(ImageArea))
{
}

TKinokoCanvasMapPlotObject::~TKinokoCanvasMapPlotObject()
{
}

TKinokoShellObject* TKinokoCanvasMapPlotObject::Clone(void)
{
    return new TKinokoCanvasMapPlotObject(_Canvas, _ImageArea);
}

float TKinokoCanvasMapPlotObject::CanvasXOf(float x)
{
    return (x - _XMin) / _XWidth * _FrameWidth + _FrameOffsetX;
}

float TKinokoCanvasMapPlotObject::CanvasYOf(float y)
{
    return (1.0 - (y - _YMin) / _YWidth) * _FrameHeight + _FrameOffsetY;
}

int TKinokoCanvasMapPlotObject::ProcessCommand(const string& Command, istream& InputStream)
{
    int Result = 0;

    if (Command == "plot") {
	Result = ProcessPlotCommand(InputStream);
    }
    else {
	Result = TKinokoCanvasFramedObject::ProcessCommand(Command, InputStream);
    }

    return Result;
}

int TKinokoCanvasMapPlotObject::ProcessPlotCommand(istream& InputStream)
{
    if (! _IsFrameValid) {
	return 1;
    }

    float Diameter;
    if (! (InputStream >> Diameter)) {
	return 0;
    }
    Diameter *= _FrameWidth / 100.0;

    int OldColorIndex = _ImageArea->SetColor(_ColorIndexList[Color_Foreground]);

    float x, y, z;
    float cx, cy;
    while (InputStream >> x >> y >> z) {
	if ((x < _XMin) || (x > _XMax) || (y < _YMin) || (y > _YMax)) {
	    continue;
	}
	cx = CanvasXOf(x);
	cy = CanvasYOf(y);

	z = clip(z, _ZMin, _ZMax);

	_ImageArea->SetColor(
	    _ColorScale->ColorIndexOf((z - _ZMin) / _ZWidth)
	);

	_ImageArea->DrawCircleFill(cx, cy, Diameter/2);
    }

    _ImageArea->SetColor(OldColorIndex);

    return 1;
}

int TKinokoCanvasMapPlotObject::DrawFrame(void)
{
    int Result = TKinokoCanvasColorScaledObject::DrawFrame();
    if (Result == 0) {
	return Result;
    }

    int OldColorIndex = _ImageArea->SetColor(_ColorIndexList[Color_Foreground]);

    _ImageArea->DrawRectFill(
	_FrameOffsetX, _FrameOffsetY, 
	_FrameOffsetX + _FrameWidth, _FrameOffsetY + _FrameHeight
    );

    _ImageArea->SetColor(OldColorIndex);

    return Result;
}
